@@ -1,10 +1,8 @@
-import base64
 import json
 import logging
-from collections import defaultdict
-from datetime import date, datetime
 from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, Union, cast, overload
 
+from .format import Formatter, decode_binary
 from .odoo_rpc import OdooClientBase, OdooModel, urljoin
 
 __doc__ = """
@@ -23,14 +21,6 @@ Get attachments and read binary values.
 Generate a report.
 """
 
-NOT_FORMATTED_FIELDS = {
-    'display_name',
-    'write_date',
-    'create_date',
-    'write_uid',
-    'create_uid',
-    '__last_update',
-}
 
 #######################################
 # ATTACHMENTS
@@ -58,52 +48,167 @@ def get_attachment(
     model: OdooModel, id: int = 0, field_name: str = None, encoded_value: str = None
 ) -> bytes:
     if encoded_value is not None:
-        return decode_bytes(encoded_value)
+        return decode_binary(encoded_value)
     if not id:
         return b''
+    # we read an attachment by id
+    if not field_name:
+        return get_attachments(model.odoo, [id])[id]
     # if we have a field name, decode the value or get the ID
-    if field_name:
-        field_info = model.fields().get(field_name, {})
-        record = model.read(id, [field_name])
-        value = record[0][field_name]
-        if field_info.get("relation") == "ir.attachment":
-            if field_info.get("type") != "many2one":
-                raise RuntimeError(
-                    "Field %s is not a many2one, here are the values: %s" % (field_name, value)
-                )
-            if not value:
-                return b''
-            value = value[0]
-        elif field_info.get("type") != "binary":
-            raise RuntimeError("%s is neither a binary or an ir.attachment field" % field_name)
-        if isinstance(value, int):
-            return get_attachment(model, value)
-        return decode_bytes(value)
-    # read the attachment
-    return get_attachments(model.odoo, [id])[0][1]
+    field_info = model.fields().get(field_name, {})
+    record = model.read(id, [field_name])
+    value = record[0][field_name]
+    if field_info.get("relation") == "ir.attachment":
+        if field_info.get("type") != "many2one":
+            raise RuntimeError(
+                "Field %s is not a many2one, here are the values: %s" % (field_name, value)
+            )
+        if not value:
+            return b''
+        value = value[0]
+    elif field_info.get("type") != "binary":
+        raise RuntimeError("%s is neither a binary or an ir.attachment field" % field_name)
+    if isinstance(value, int):
+        return get_attachment(model, value)
+    return decode_binary(value)
 
 
-def decode_bytes(value) -> bytes:
-    return base64.b64decode(value)
-
-
-def get_attachments(odoo: OdooClientBase, ids: List[int]) -> List[Tuple[str, bytes]]:
+def get_attachments(odoo: OdooClientBase, ids: List[int]) -> Dict[int, bytes]:
     """Get a list of tuples (name, raw_bytes) from ir.attachment by ids
 
     :param odoo: Odoo client
     :param ids: List of ids of attachments
-    :return: List of tuples (name, raw_bytes)
+    :return: Dict id -> raw_bytes
     """
-    data = odoo['ir.attachment'].read(ids, ['name', 'datas'])
-    return [(r['name'], decode_bytes(r['datas'])) for r in data]
+    data = odoo['ir.attachment'].read(ids, ['datas'])
+    return {r['id']: decode_binary(r['datas']) for r in data}
+
+
+def list_attachments(
+    model: OdooModel, ids: List[int], *, domain=[], fields=[], generate_access_token=False
+) -> List[Dict]:
+    """List attachments
+
+    We search all attachments linked to the model and ids.
+    All attachments, no matter if they have a res_field or not are read.
+    We read the id, name, res_id, res_field.
+    Special field names: public_url (generate public URL), datas (contents).
+
+    :param model: The model
+    :param ids: The record ids to filter on
+    :param domain: Additional domain
+    :param fields: Additional fields to read
+    :param generate_access_token: Generate access token for the found documents
+    :return: List of read properties from the attachments (id, name, res_field, ...)
+    """
+    url_field = None
+    if 'public_url' in fields:
+        url_field = 'public_url'
+        fields += ['website_url', 'url', 'public', 'access_token']
+    fields = list({'id', 'name', 'res_id', 'res_field', *fields} - {url_field})
+    # Get the data
+    if model.model == 'ir.attachment':
+        attachments = model
+        data = attachments.search_read(
+            [
+                ('id', 'in', ids),
+            ]
+            + domain,
+            fields,
+        )
+    else:
+        attachments = model.odoo['ir.attachment']
+        data = attachments.search_read(
+            [
+                ('res_model', '=', model.model),
+                ('res_id', 'in', ids),
+                ('id', '!=', 0),  # to get all res_field
+            ]
+            + domain,
+            fields,
+        )
+    # Get contents
+    if 'datas' in fields:
+        for d in data:
+            d['datas'] = decode_binary(d['datas']) if d['datas'] else False
+    # Generate access tokens
+    if generate_access_token and data:
+        tokens = attachments.execute('generate_access_token', [d['id'] for d in data])
+        for d, token in zip(data, tokens):
+            d['access_token'] = token
+    # Compute a full url (false when not accessible)
+    if url_field:
+
+        def attachment_url(d, _model_name, _id):
+            url = d.get('url') or d.get('website_url')
+            if url and d.get('public'):
+                return url
+            if url and d.get('access_token'):
+                url += ('&' if '?' in url else '?') + 'access_token=' + d.get('access_token')
+                return url
+            return False
+
+        add_url(attachments, data, url_field=url_field, build_url=attachment_url)
+    return data
+
+
+def _download_content(
+    odoo: OdooClientBase, url: str, *, params: Dict = {}, access_token: Optional[str] = None
+) -> bytes:
+    """Download contents from a URL"""
+    # In the implementation, the session is authenticated.
+    session = getattr(odoo, 'session', None)
+    if not session:
+        raise RuntimeError('Odoo client must have a session to download contents')
+
+    url = urljoin(odoo.url, url)
+    params = {}
+    if access_token:
+        params = {**params, 'access_token': access_token}
+    req = session.get(url, params=params)
+    req.raise_for_status()
+    return req.content
+
+
+def download_content(
+    model: OdooModel,
+    id: int,
+    field_name: Optional[str] = None,
+    *,
+    access_token: Optional[str] = None,
+) -> bytes:
+    """Download contents from /web"""
+    if field_name:
+        url = f"/web/content/{model.model}/{id}/{field_name}"
+    else:
+        url = f"/web/content/{id}"
+    return _download_content(model.odoo, url, access_token=access_token)
+
+
+def download_image(
+    model: OdooModel,
+    id: int,
+    field_name: Optional[str] = None,
+    *,
+    dimensions=None,
+    access_token: Optional[str] = None,
+):
+    """Download an image from /web"""
+    if field_name:
+        url = f"/web/image/{model.model}/{id}/{field_name}"
+    else:
+        url = f"/web/image/{id}"
+    if dimensions:
+        url += "/%dx%d" % dimensions
+    return _download_content(model.odoo, url, access_token=access_token)
 
 
 #######################################
 # REPORTS
 
 
-def get_reports(model: OdooModel) -> List[Dict]:
-    """Get a list of reports from a model"""
+def list_reports(model: OdooModel) -> List[Dict]:
+    """List of reports from a model"""
     return model.odoo['ir.actions.report'].search_read(
         [('model', '=', model.model)], ['name', 'report_type', 'report_name']
     )
@@ -118,106 +223,18 @@ def get_report(model: OdooModel, report_name: str, id: int, converter='pdf') ->
     :param converter: The output type (default: pdf)
     :return: Bytes of the report
     """
-    from .odoo_rpc_json import OdooClientJSON, urljoin
-
     # We need to use the session, because there is no public API since v14 to
     # render a report and get the bytes.
-    # In the implementation, the session is authenticated.
-    if not isinstance(model.odoo, OdooClientJSON):
-        raise RuntimeError('Can get the report only using OdooClientJSON')
-
     if converter.startswith('qweb-'):
         converter = converter[5:]
-    url = urljoin(model.odoo.url, f"/report/{converter}/{report_name}/{id}")
-    req = model.odoo.session.get(url)
-    req.raise_for_status()
-    return req.content
+    url = f"/report/{converter}/{report_name}/{id}"
+    return _download_content(model.odoo, url)
 
 
 #######################################
 # IMPORT AND EXPORT
 
 
-def format_default(v: Any):
-    """Format a value for Odoo"""
-    if isinstance(v, str):
-        v = v.strip()
-    return v or False
-
-
-def format_datetime(v: Union[datetime, str]):
-    """Format a datetime to insert into Odoo"""
-    if isinstance(v, datetime):
-        # format ISO with a space an ignore millisenconds
-        return v.isoformat(sep=" ", timespec='seconds')
-    if isinstance(v, str):
-        return v.strip()[:19] or False
-    return False
-
-
-def format_date(v: Union[datetime, date, str]):
-    """Format a date to insert into Odoo"""
-    if isinstance(v, datetime):
-        v = v.date()
-    if isinstance(v, date):
-        return v.isoformat()
-    if isinstance(v, str):
-        return v.strip()[:10] or False
-    return v
-
-
-def format_binary(v: Union[bytes, str]) -> str:
-    """Format bytes to insert into Odoo (as base64)"""
-    if isinstance(v, str):
-        v = bytes(v, 'utf-8')
-    encoded = base64.b64encode(v)
-    return str(encoded, 'ascii')
-
-
-class Formatter(defaultdict):
-    """Format fields into a format expected by Odoo."""
-
-    """Transformations to apply to source fields.
-    Use an empty string to mask some of them."""
-    field_map: Dict[str, str]
-
-    def __init__(self, model: OdooModel = None, *, lower_case_fields: bool = False):
-        """New formatter
-
-        :param model: The model for which the formatter is generated (initializes formatters)
-        :param lower_case_fields: Whether to transform source fields into lower-case
-        """
-        super().__init__(lambda: format_default)
-        self.model = model
-        self.lower_case_fields = lower_case_fields
-        self.field_map = {}
-
-        # set the formats from the model
-        if model:
-            formatters = {
-                'datetime': format_datetime,
-                'date': format_date,
-                'binary': format_binary,
-                'image': format_binary,
-            }
-            for field, info in model.fields().items():
-                formatter = formatters.get(cast(str, info.get('type')), format_default)
-                if formatter is not format_default:
-                    self.setdefault(field, formatter)
-
-    def map_field(self, name):
-        """Transform a source field name into model's name"""
-        name = self.field_map.get(name, name)
-        if self.lower_case_fields:
-            name = name.lower()
-        return name
-
-    def format_dict(self, d: Dict[str, Any]) -> Dict[str, Any]:
-        """Apply formatting to each field"""
-        d_renamed = [(self.map_field(f), v) for f, v in d.items()]
-        return {f: self[f](v) for f, v in d_renamed if f and f not in NOT_FORMATTED_FIELDS}
-
-
 def make_batches(
     data: Iterable[Dict], *, batch_size: int = 1000, group_by: str = None
 ) -> Iterable[List[Dict]]:
@@ -296,7 +313,7 @@ def load_data(
         data = [formatter.format_dict(d) for d in data]
     elif method_row_type == list:
         data, fields = __convert_to_type_list(data, fields)
-        data = [[formatter[fields[i]](v) for i, v in enumerate(d)] for d in data]
+        data = [[formatter.format_function[fields[i]](v) for i, v in enumerate(d)] for d in data]
     else:
         raise Exception('Unsupported method row type: %s' % method_row_type)
 
@@ -514,13 +531,21 @@ def add_xml_id(model: OdooModel, data: List, *, id_name='id', xml_id_field='xml_
                 row.append(xml_ids.get(row[id_index], False))
 
 
-def add_url(model: OdooModel, data: List, *, model_id_func=None, url_field='url'):
+def add_url(
+    model: OdooModel,
+    data: List,
+    *,
+    url_field='url',
+    model_id_func=None,
+    build_url=lambda _r, model_name, id: f"/web#model={model_name}&id={id}",
+):
     """Add an URL field to data
 
     :param model: The model
     :param data: The list of dicts or rows (list to append to)
-    :param model_id_func: The function to get a tuple (model_name, id)
     :param url_field: The name of the URL field (default: 'url')
+    :param model_id_func: The function to get a tuple (model_name, id)
+    :param build_url: Function to build the URL (record, model_name, id) -> "/web..."
     """
     base_url = model.odoo.url
     if not model_id_func and data:
@@ -530,14 +555,16 @@ def add_url(model: OdooModel, data: List, *, model_id_func=None, url_field='url'
             model_id_func = lambda d: (model.model, d['id'])  # noqa: E731
     for row_num, row in enumerate(data):
         model_name, id = model_id_func(row)
-        url = f"/web#model={model_name}&id={id}"
+        url = build_url(row, model_name, id)
+        if url:
+            url = urljoin(base_url, url)
         if isinstance(row, dict):
-            row[url_field] = urljoin(base_url, url.format(id=id))
+            row[url_field] = url
         elif isinstance(row, list):
             if row_num == 0:
                 row.append(url_field)
             else:
-                row.append(urljoin(base_url, url.format(id=id)))
+                row.append(url)
         else:
             raise TypeError('Cannot append the url to %s' % type(row))
 
@@ -595,8 +622,9 @@ def flatten(data: List[Dict], fields: List[str], expand_many=False) -> List[List
 __all__ = [
     'get_attachment',
     'get_attachments',
+    'list_attachments',
     'get_report',
-    'get_reports',
+    'list_reports',
     'Formatter',
     'make_batches',
     'load_data',
