@@ -1,4 +1,5 @@
 import odoo_connect.data as odoo_data
+from odoo_connect.format import format_binary
 
 
 def test_get_attachment(odoo_session):
@@ -16,13 +17,10 @@ def test_get_attachment_field(odoo_session):
 
 def test_get_attachment_bytes(odoo_session):
     bin = bytes("hello", 'ascii')
-    value = odoo_data.get_attachment(
-        odoo_session['res.partner'], encoded_value=odoo_data.format_binary(bin)
-    )
+    value = odoo_data.get_attachment(odoo_session['res.partner'], encoded_value=format_binary(bin))
     assert bin == value
 
 
 def test_get_attachments(odoo_session):
     values = odoo_data.get_attachments(odoo_session, [1, 2])
-    assert len(values) == 2
-    assert all(isinstance(name, str) and isinstance(raw, bytes) for name, raw in values)
+    assert isinstance(values, dict) and values
